#------------------------------------------------------------------------------
# Copyright 2008-2012 Istituto Nazionale di Fisica Nucleare (INFN)
# 
# Licensed under the EUPL, Version 1.1 only (the "Licence").
# You may not use this work except in compliance with the Licence. 
# You may obtain a copy of the Licence at:
# 
# http://joinup.ec.europa.eu/system/files/EN/EUPL%20v.1.1%20-%20Licence.pdf
# 
# Unless required by applicable law or agreed to in 
# writing, software distributed under the Licence is 
# distributed on an "AS IS" basis, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. 
# See the Licence for the specific language governing 
# permissions and limitations under the Licence.
#------------------------------------------------------------------------------

import os
import commands
import exceptions
from wnodes.cli.utils import load_config
from wnodes.cli.utils import checks
from wnodes.cli.utils import server_errors
from wnodes.cli.utils import user
from wnodes.cli.commands import errors
from wnodes.cli.commands import options

class ListInstancesError(exceptions.Exception):
    pass

class ListInstances(object):
    def __init__(self, user_data, path = '/-/'):
        self.path = path
        self.cmd_name = checks.get_cmd_name('curl')
        self.user_data = user_data
        self.set_cmd = options.CmdOptions(self.user_data)

    def get_command(self, endpoint, provider, not_show=True):
        '''Gets command string'''
        cmd_general_options = self.set_cmd.general(not_show=not_show)
        cmd_auth_options = self.set_cmd.auth()
        cmd_provider_option = self.set_cmd.provider_option(provider, endpoint)

        uri = ('%s://%s:%s/%s/'
           % (self.user_data[endpoint]['protocol'],
           endpoint, self.user_data[endpoint]['port'], self.path))

        cmd = (self.cmd_name + cmd_general_options +
           cmd_auth_options +
           cmd_provider_option + uri)

        return cmd

    def run_command(self, endpoint, provider, not_show=True):
        '''Runs command'''
        cmd_output = commands.getstatusoutput(self.get_command(endpoint, provider, not_show=not_show))
        return cmd_output

    def get_output(self, endpoint, provider, not_show=True):
        '''Returns Images Information'''
        output = self.run_command(endpoint, provider, not_show=not_show)

        correct_values = []
        msg = ''

        if len(output) > 0 and output[0] == 0:
            values = output[1].split('\n')

            errors = server_errors.ServerErrors()
            for x in values:
                if x in errors.get_errors():
                    msg = x

            if msg == '':
                for y in errors.get_occi_errors():
                    if y in values[0]:
                        raise ListInstancesError(values[0].split(y)[1])
                for x in values[5:]:
                    correct_values.append(str(x))

        elif output[0] > 0:
            output = self.run_command(endpoint, provider, not_show=False)
            raise ListInstancesError(output[1][1:])

        return correct_values, msg
