#------------------------------------------------------------------------------
# Copyright 2008-2012 Istituto Nazionale di Fisica Nucleare (INFN)
# 
# Licensed under the EUPL, Version 1.1 only (the "Licence").
# You may not use this work except in compliance with the Licence. 
# You may obtain a copy of the Licence at:
# 
# http://joinup.ec.europa.eu/system/files/EN/EUPL%20v.1.1%20-%20Licence.pdf
# 
# Unless required by applicable law or agreed to in 
# writing, software distributed under the Licence is 
# distributed on an "AS IS" basis, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. 
# See the Licence for the specific language governing 
# permissions and limitations under the Licence.
#------------------------------------------------------------------------------

import os
import commands
import exceptions
from cStringIO import StringIO

from wnodes.cli.utils import load_config
from wnodes.cli.utils import user
from wnodes.cli.commands import auth_openstack
from wnodes.cli.commands import errors

class CmdOptionsError(exceptions.Exception):
    pass

class CmdOptions(object):

    def __init__(self, user_data, image_data='', image_id=''):

        self.image_data = image_data
        self.user_data = user_data
        self.image_id = image_id

    def __get_line(self, filename):
        in_file = open(filename,"r")
        text = in_file.read()
        in_file.close()
        return text.split('\n')[0]

    def __set_vo(self):
        if 'vo' in self.image_data.keys():
            return self.image_data['vo']

        return self.user_data['user']['vo']

    def __get_id(self, endpoint):
        token = auth_openstack.Token(self.user_data[endpoint]['auth_path'],
            endpoint)
        try:
            id=token.get_id()
        except auth_openstack.TokenExpired, err:
            auth = auth_openstack.AuthOpenStack(self.user_data)
            auth.perform_a_transfer(endpoint)
            token.store_token(auth.get_token())
            id=auth.get_id()
        except IOError, err:
            auth = auth_openstack.AuthOpenStack(self.user_data)
            auth.perform_a_transfer(endpoint)
            token.store_token(auth.get_token())
            id=auth.get_id()
        return id

    def provider_option(self, provider, endpoint):
        '''Sets provider option'''
        if provider.lower() == 'wnodes':
            provider_option = ("-H 'Accept: %s' "
                % self.user_data['content']['type'])
        elif provider.lower() == 'openstack':
            provider_option = ("-H 'Content-Type: %s' "
                % self.user_data['content']['type'] +
                "-H 'X-Auth-Token: %s' "
                % self.__get_id(endpoint))
        elif provider.lower() == 'opennebula':
            provider_option = ''
        else:
            msg = 'The specified providers in the data are wrong'
            raise errors.DataError(msg)

        return provider_option

    def provider_options(self, provider, endpoint):
        '''Sets provider options'''
        if provider.lower() == 'wnodes':
            provider_options = ("-H 'X-Auth-VO: %s' "
                % self.__set_vo() +
                "-H 'X-Ssh-Pk: %s' "
                % self.__get_line(self.user_data['user']['pub_key']) +
                "-H 'Category: compute; " +
                'scheme="http://schemas.ogf.org/occi/infrastructure#"; ' +
                'class="kind";'+ "' " +
                "-H 'Content-Type: %s' "
                % self.user_data['content']['type'] +
                "-H 'Category: %s; "
                % self.image_data['size'] +
                'scheme="http://schemas.wnodes.org/template/resource#"; ' +
                'class="mixin";'+ "' " +
                "-H 'Category: %s; "
                % self.image_id[0] +
                'scheme="http://schemas.wnodes.org/template/os#"; ' +
                'class="mixin";'+ "' ")
        elif provider.lower() == 'openstack':
            provider_options = ("-H 'Category: compute; " +
                'scheme="http://schemas.ogf.org/occi/infrastructure#"; ' +
                'class="kind";'+ "' " +
                "-H 'X-Auth-Token: %s' " 
                % self.__get_id(endpoint) +
                "-H 'Content-Type: %s' "
                % self.user_data['content']['type'] +
                "-H 'Category: %s; "
                % self.image_data['size'] +
                'scheme="http://schemas.openstack.org/template/resource#"; ' +
                'class="mixin";'+ "' " +
                "-H 'Category: %s; "
                % self.image_id[0] +
                'scheme="http://schemas.openstack.org/template/os#"; ' +
                'class="mixin";'+ "' "
                )
        elif provider.lower() == 'opennebula':
            #TODO set size
            provider_options = ("-H 'Category: compute; " +
                'scheme="http://schemas.ogf.org/occi/infrastructure#"; ' +
                'class="kind";'+ "' " +
                "-H 'Category: %s; "
                % self.image_data['size'] +
                'scheme="http://schemas.opennebula.org/template/resource#"; ' +
                'class="mixin";'+ "' " +
                "-H 'Category: %s; "
                % self.image_id[0] +
                'scheme="http://schemas.opennebula.org/template/os#"; ' +
                'class="mixin";'+ "' "
                )
                #"-H 'Link: <%s>; "
                #% self.image_data["requires"].replace(endpoint,'') +
                #'rel="http://schemas.ogf.org/occi/infrastructure#network"; ' +
                #'category="http://schemas.ogf.org/occi/infrastructure#networkinterface";, ' +
                #"<%s>; " 
                #% self.image_data["location"].replace(endpoint,'') +
                #'rel="http://schemas.ogf.org/occi/infrastructure#storage"; ' +
                #'category="http://schemas.ogf.org/occi/infrastructure#storagelink"; '
        else:
            msg = 'The specified providers in the data are wrong'
            raise errors.DataError(msg)

        return provider_options

    def general(self, not_show):
        '''Sets cmd general option'''
        not_show_progress_meter = ' '
        if not_show == True:
            not_show_progress_meter = ' -s '
        info_option = ' -i '

        return '%s %s' % (not_show_progress_meter, info_option)

    def auth(self):
        '''Sets cmd auth options'''
        map_value = load_config.UserParameters().get_map()
        cacert = ''
        cert,key = user.set_cert_key(data=self.user_data['user'])
        if not '/' in cert:
            cert=os.getcwd()+'/'+cert
        if not '/' in key:
            key=os.getcwd()+'/'+key
        if cert == key:
            cacert = cert
        cert_option = ('%s %s '
            % (map_value['user_cert'], cert))
        key_option = ('%s %s '
            % (map_value['user_key'], key))
        cacert_option = ''
        if cacert != '':
            cacert_option = ('--cacert %s '
                % cacert )
        capath_option = ('%s %s '
            % (map_value['ca_path'], self.user_data['user']['ca_path']))

        return ('%s %s %s %s' 
            % (cacert_option, capath_option, cert_option, key_option))
