import exceptions
import urllib2
import re
from wnodes.cli.utils import utilities

from stratuslab.Exceptions import InputException
from stratuslab.Exceptions import ExecutionException
from stratuslab.ManifestInfo import ManifestInfo
import stratuslab.Util as Util

class MetadataInfo(object):
    def __init__(self, data):
        self.uri = data['marketplace']['uri']
        self.ns_dcterms = data['marketplace']['ns_dcterms']
        self.etree = Util.importETree()
        self.providers = data['providers']

    def __parseXml(self, xmlAsString):
        return self.etree.fromstring(xmlAsString)

    def __set_platform(self, location):
        platform = ''
        for key, value in self.providers.iteritems():
            for endpoint in value.replace(" ","").split(','):
                if endpoint in location:
                    if key.lower() == 'opennebula':
                        platform = 'OpenNebula'
                    elif key.lower() == 'openstack':
                        platform = 'OpenStack'
                    elif key.lower() == 'wnodes':
                        platform = 'WNoDeS'
                    elif key.lower() == 'wnodes':
                       if 'wnodesrepo' in location:
                            platform = 'WNoDeS'
                elif key.lower() == 'wnodes':
                    if 'wnodesrepo' in location:
                         platform = 'WNoDeS'
        return platform

    def __extractMetadataInfos(self, manifestRootElement):
        manifestElements = manifestRootElement.findall('{http://www.w3.org/1999/02/22-rdf-syntax-ns#}RDF')
        manifests = []
        infos = {}
        for element in manifestElements:
            manifest = ManifestInfo()
            manifest.parseManifestFromXmlTree(element)
            if len(manifest.locations) > 0:
                info = {
                    'md5': manifest.md5,
                    'title': manifest.title,
                    'os': manifest.os,
                    'os-version':manifest.osversion,
                    'os-arch':manifest.arch,
                    'location':manifest.locations[0],
                    'creator':manifest.creator,
                    'valid':manifest.valid.replace("T"," "),
                    'description':manifest.comment,
                    'publisher':manifest.publisher,
                    'format':manifest.format,
                    'hypervisor':manifest.hypervisor,
                    'platform': self.__set_platform(manifest.locations[0])
                }
                #requires = getattr(element.find('.//{%s}requires' % self.ns_dcterms), 'text', '')
                #if requires:
                #    info['requires'] = requires
                #    if manifest.locations[0] == requires:
                #        pat = re.compile(r'http[s]{0,1}://[a-z].[a-z][a-z\.\-0-9]*:[0-9]+[a-z.0-9/\-]')
                #        domain = re.findall(pat,manifest.locations[0])
                #        info['identifier'] = manifest.locations[0].replace(domain[0],"").replace("/","")
                print info['location']
                if info['platform'] != '':
                    infos[(manifest.identifier, manifest.endorser, manifest.created)] = info
        return infos

    def get_metadata_info(self):
        metadataEntries = ''
        try:
            metadataEntries = Util.wstring(self.uri)
        except urllib2.HTTPError:
            msg = 'Failed to find metadata entries: %s' % self.uri
            raise InputException(msg)
        except urllib2.URLError:
            raise InputException('Wrong URL: %s' % self.uri)
        return self.__extractMetadataInfos(self.__parseXml(metadataEntries))

    def print_metadata_info(self, format=()):
        meta = self.get_metadata_info()
        if format[0]:
            self.__print_info_with_custom_format(meta, format[1])
        else:
            self.__print_info_with_default_format(meta)

    def __print_info_with_custom_format(self, meta, format):
        title = []
        os = []
        os_version = []
        os_arch = []
        endorser = []
        identifier = []
        platform = []
        des = []
        publisher = []

        for key, value in meta.iteritems():
            title.append(value['title'])
            os.append(value['os'])
            os_version.append(value['os-version'])
            os_arch.append(value['os-arch'])
            endorser.append(key[1])
            identifier.append(key[0])
            platform.append(value['platform'])
            des.append(value['description'])
            publisher.append(value['publisher'])

        title_max_length,longest_title = utilities.get_longest_string(title)
        os_max_length,longest_os = utilities.get_longest_string(os)
        osv_max_length,longest_osv = utilities.get_longest_string(os_version)
        osa_max_length,longest_osa = utilities.get_longest_string(os_arch)
        en_max_length,longest_en = utilities.get_longest_string(endorser)
        id_max_length,longest_id = utilities.get_longest_string(identifier)
        platform_max_length,longest_platform = \
            utilities.get_longest_string(platform)
        pub_max_length,longest_pub = utilities.get_longest_string(publisher)
        d_max_length,longest_d = utilities.get_longest_string(des)

        custom_headline = ''
        for entry in format:
            if entry[0].lower() == 'title':
                custom_headline += ('%s%s '
                    % (entry[0],
                    utilities.add_empty_space(len('TITLE'),
                    title_max_length))) 
            elif entry[0].lower() == 'platform':
                custom_headline += ('%s%s '
                    % (entry[0], utilities.add_empty_space(len('PLATFORM'),
                    platform_max_length)))
            elif entry[0].lower() == 'publisher':
                custom_headline += ('%s%s '
                    % (entry[0], utilities.add_empty_space(len('PUBLISHER'),
                    pub_max_length)))
            elif entry[0].lower() == 'os':
                custom_headline += ('%s%s '
                    % (entry[0], utilities.add_empty_space(len('OS'),
                    os_max_length)))
            elif entry[0].lower() == 'os-version':
                custom_headline += ('%s%s '
                    % (entry[0], utilities.add_empty_space(len('OS-VERSION'),
                    osv_max_length)))
            elif entry[0].lower() == 'os-arch':
                custom_headline += ('%s%s '
                    % (entry[0], utilities.add_empty_space(len('OS-ARCH'),
                    osa_max_length)))
            elif entry[0].lower() == 'endorser':
                custom_headline += ('%s%s '
                    % (entry[0], utilities.add_empty_space(len('ENDORSER'),
                    en_max_length)))
            elif entry[0].lower() == 'identifier':
                custom_headline += ('%s%s '
                    % (entry[0], utilities.add_empty_space(len('IDENTIFIER'),
                    id_max_length)))
            elif entry[0].lower() == 'description':
                custom_headline += ('%s%s '
                    % (entry[0], utilities.add_empty_space(len('DESCRIPTION'),
                    d_max_length)))
        print custom_headline

        for key, value in meta.iteritems():
            new_line = ''
            for entry in format:
                if entry[0].lower() == 'title':
                    new_line += ('%s%s '
                        % (value['title'],
                        utilities.add_empty_space(len(value['title']),
                        title_max_length)))
                elif entry[0].lower() == 'platform':
                    new_line += ('%s%s '
                        % (value['platform'],
                        utilities.add_empty_space(len(value['platform']),
                        platform_max_length)))
                elif entry[0].lower() == 'publisher':
                    new_line += ('%s%s '
                        % (value['publisher'],
                        utilities.add_empty_space(len(value['publisher']),
                        pub_max_length)))
                elif entry[0].lower() == 'os':
                    new_line += ('%s%s '
                        % (value[0],
                        utilities.add_empty_space(len(value['os']),
                        os_max_length)))
                elif entry[0].lower() == 'os-version':
                    new_line += ('%s%s '
                        % (value[0],
                        utilities.add_empty_space(len(value['os-version']),
                        osv_max_length)))
                elif entry[0].lower() == 'os-arch':
                    new_line += ('%s%s '
                        % (value[0],
                        utilities.add_empty_space(len(value['os-arch']),
                        osa_max_length)))
                elif entry[0].lower() == 'endorser':
                    new_line += ('%s%s '
                        % (key[1],
                        utilities.add_empty_space(len(key[1]),
                        en_max_length)))
                elif entry[0].lower() == 'identifier':
                    new_line += ('%s%s '
                        % (key[0],
                        utilities.add_empty_space(len(key[0]),
                        id_max_length)))
                elif entry[0].lower() == 'description':
                    new_line += ('%s%s '
                        % (value['description'],
                        utilities.add_empty_space(len(value['description']),
                        d_max_length)))
            print new_line
    
    def __print_info_with_default_format(self, meta):    
        title = []
        os = []
        os_version = []
        os_arch = []
        endorser = []
        identifier = []
        platform = []
        publisher = []       

        for key, value in meta.iteritems():
            title.append(value['title'])
            os.append(value['os'])
            os_version.append(value['os-version'])
            os_arch.append(value['os-arch'])
            endorser.append(key[1])
            identifier.append(key[0])
            platform.append(value['platform'])
            publisher.append(value['publisher'])

        title_max_length,longest_title = utilities.get_longest_string(title)
        os_max_length,longest_os = utilities.get_longest_string(os)
        osv_max_length,longest_osv = utilities.get_longest_string(os_version)
        osa_max_length,longest_osa = utilities.get_longest_string(os_arch)
        en_max_length,longest_en = utilities.get_longest_string(endorser)
        id_max_length,longest_id = utilities.get_longest_string(identifier)
        platform_max_length,longest_platform = \
            utilities.get_longest_string(platform)
        pub_max_length,longest_pub = utilities.get_longest_string(publisher)

        print ('TITLE%s PLATFORM%s OS%s OS-VERSION%s OS-ARCH%s IDENTIFIER%s '
            % (utilities.add_empty_space(len('TITLE'), title_max_length),
            utilities.add_empty_space(len('PLATFORM'), platform_max_length),
            utilities.add_empty_space(len('OS'), os_max_length),
            utilities.add_empty_space(len('OS-VERSION'), osv_max_length),
            utilities.add_empty_space(len('OS-ARCH'), osa_max_length),
            utilities.add_empty_space(len('IDENTIFIER'), id_max_length)) +
            'ENDORSER%s PUBLISHER%s DESCRIPTION'
            % (utilities.add_empty_space(len('ENDORSER'), en_max_length),
            utilities.add_empty_space(len('PUBLISHER'), pub_max_length)))
        for key, value in meta.iteritems():
            print ('%s%s %s%s %s%s %s%s %s%s  %s%s %s%s %s%s %s'
                % (value['title'],
                utilities.add_empty_space(len(value['title']),
                title_max_length),
                value['platform'],
                utilities.add_empty_space(len(value['platform']),
                platform_max_length),
                value['os'],
                utilities.add_empty_space(len(value['os']),
                os_max_length),
                value['os-version'],
                utilities.add_empty_space(len(value['os-version']),
                osv_max_length),
                value['os-arch'],
                utilities.add_empty_space(len(value['os-arch']),
                osa_max_length),
                key[0], utilities.add_empty_space(len(key[0]), id_max_length),
                key[1], utilities.add_empty_space(len(key[1]), en_max_length),
                value['publisher'],
                utilities.add_empty_space(len(value['publisher']),
                pub_max_length),
                value['description']))
