import ldap
import exceptions

class SearchLdapError(exceptions.Exception):
    pass

class SearchProviders():

    basedn = 'GLUE2GroupID=cloud,o=glue'
    attributes = ['GLUE2EndpointURL',
        'GLUE2EndpointImplementationName',
        'GLUE2EndpointInterfaceName']
    filter = 'GLUE2EndpointID=*'

    def __init__(self, server, port):
        self.server = server
        self.port = port

        self.result = {
            'WNoDeS': [],
            'OpenNebula': [],
            'OpenStack': []
            }

    def query(self):
        uri = ('ldap://%s:%s'
            % (self.server, self.port))

        ldap_init = ldap.initialize(uri)

        search_scope = ldap.SCOPE_SUBTREE
        result_set = []

        try:
            ldap_result_id = ldap_init.search(SearchProviders.basedn,
                search_scope,
                SearchProviders.filter,
                SearchProviders.attributes)

            while 1:
                result_type, result_data = ldap_init.result(ldap_result_id, 0)

                if (result_data == []):
                    break
                else:
                    ## here you don't have to append to a list
                    ## you could do whatever you want with the individual entry
                    ## The appending to list is just for illustration. 
                    if result_type == ldap.RES_SEARCH_ENTRY:
                        result_set.append(result_data)

        except ldap.CONNECT_ERROR, e:
            if 'desc' in e[0].keys():
                raise SearchLdapError(e[0]['desc'])
            else:
                raise SearchLdapError(e[0])
        except ldap.SERVER_DOWN, e:
            if 'desc' in e[0].keys():
                raise SearchLdapError(e[0]['desc'])
            else:
                raise SearchLdapError(e[0])
        except ldap.TIMEOUT, e:
            if 'desc' in e[0].keys():
                raise SearchLdapError(e[0]['desc'])
            else:
                raise SearchLdapError(e[0])
        except ldap.BUSY, e:
            if 'desc' in e[0].keys():
                raise SearchLdapError(e[0]['desc'])
            else:
                raise SearchLdapError(e[0])
        except ldap.OPT_NETWORK_TIMEOUT, e:
            if 'desc' in e[0].keys():
                raise SearchLdapError(e[0]['desc'])
            else:
                raise SearchLdapError(e[0])
        
        return result_set

    def parse_query(self):
        output=self.query()
        if len(output) > 0:
            for y in output:
                for z in y:
                    for element in z:
                        if type(element) is dict:
                            if SearchProviders.attributes[1] in element.keys() and \
                                element[SearchProviders.attributes[1]][0] in self.result.keys():
                                if SearchProviders.attributes[0] in element.keys() and \
                                    element[SearchProviders.attributes[2]][0] == 'OCCI':
                                    self.result[element[SearchProviders.attributes[1]][0]].append(element[SearchProviders.attributes[0]][0])

    def print_providers_info(self):
        self.parse_query()
        for provider, values in self.result.iteritems():
            print ("Endpoint:Port of the %s Resource Provider:"
                % provider)
            for endpoint in values:
                if 'https' in endpoint:
                    print endpoint.split('://')[1].split('/')[0]

            print "\n" 
