import unittest
import inspect
from wnodes.cli.utils import load_config

class UserParametersTestCase(unittest.TestCase):
    parameters = {
        'marketplace': {'uri':'',
            'ns_dcterms':''},
        'providers': {
            'wnodes':'',
            'openstack':'',
            'opennebula':''},
        'user': {'vo': '',
            'user_key': '',
            'user_cert': '',
            'user_proxy': '',
            'ca_path': '/etc/grid-security/certificates'},
        'content': {'type': ''},
        'infrastructure': {'category':''}
        }

    providers = {'wnodes': ['port', 'ns_port', 'ns_host'],
        'openstack': ['port'],
        'opennebula': ['port']
        }

    cmd_options = ['--capath',
        '--cert',
        '--key']

    def test_get_parameters(self):
        stack_value = inspect.stack()[0]
        path = stack_value[1]
        method = stack_value[3]

        try:
            up = load_config.UserParameters()
            parameters = up.get_parameters()
            msg = 'Wrong User Config Parameters'
            self.assert_(parameters.keys() not in UserParametersTestCase.parameters.keys(), \
                '%s, %s - FAILED, %s' %
                (path, method, msg))
        except AssertionError, err:
            print err

    def test_get_providers(self):
        stack_value = inspect.stack()[0]
        path = stack_value[1]
        method = stack_value[3]

        try:
            up = load_config.UserParameters()
            providers = up.get_providers()
            msg = 'Wrong User Config Providers'
            self.assert_(providers.keys() not in UserParametersTestCase.providers.keys(), \
                '%s, %s - FAILED, %s' %
                (path, method, msg))
        except AssertionError, err:
            print err

    def test_get_wnodes(self):
        stack_value = inspect.stack()[0]
        path = stack_value[1]
        method = stack_value[3]

        try:
            up = load_config.UserParameters()
            providers = up.get_providers('wnodes')
            msg = 'Wrong User Config WNoDeS Provider'
            self.assert_(providers not in UserParametersTestCase.providers['wnodes'], \
                '%s, %s - FAILED, %s' %
                (path, method, msg))
        except AssertionError, err:
            print err

    def test_get_openstack(self):
        stack_value = inspect.stack()[0]
        path = stack_value[1]
        method = stack_value[3]

        try:
            up = load_config.UserParameters()
            providers = up.get_providers('openstack')
            msg = 'Wrong User Config OpenStack Provider'
            self.assert_(providers not in UserParametersTestCase.providers['openstack'], \
                '%s, %s - FAILED, %s' %
                (path, method, msg))
        except AssertionError, err:
            print err

    def test_get_opennebula(self):
        stack_value = inspect.stack()[0]
        path = stack_value[1]
        method = stack_value[3]

        try:
            up = load_config.UserParameters()
            providers = up.get_providers('opennebula')
            msg = 'Wrong User Config Opennebula Provider'
            self.assert_(providers not in UserParametersTestCase.providers['opennebula'], \
                '%s, %s - FAILED, %s' %
                (path, method, msg))
        except AssertionError, err:
            print err

    def test_get_others(self):
        stack_value = inspect.stack()[0]
        path = stack_value[1]
        method = stack_value[3]

        try:
            up = load_config.UserParameters()
            providers = up.get_providers('new')
            msg = 'Wrong User Config New Provider'
            self.assert_(providers not in UserParametersTestCase.providers['new'], \
                '%s, %s - FAILED, %s' %
                (path, method, msg))
        except load_config.WrongUserParameters, err:
            print err
        except AssertionError, err:
            print err

    def test_get_map(self):
        stack_value = inspect.stack()[0]
        path = stack_value[1]
        method = stack_value[3]

        try:
            up = load_config.UserParameters()
            map = up.get_map()
            msg = 'Wrong User Config Options'
            self.assert_(map.values not in UserParametersTestCase.cmd_options, \
                '%s, %s - FAILED, %s' %
                (path, method, msg))
        except load_config.WrongUserParameters, err:
            print err
        except AssertionError, err:
            print err

def main():
    suite = unittest.TestLoader().loadTestsFromTestCase(UserParametersTestCase)
    unittest.TextTestRunner(verbosity=2).run(suite)

if __name__ == '__main__':
    main()
